/*
 * File:   main.c
 * Author: tobias poppe
 * Use the UART and the ADC of the Fab-Pic
 *
 * Created on April 19, 2016, 10:24 AM
 */

#include <xc.h>
#include <plib.h>
#include <adc.h>
#include <stdio.h>


#define _XTAL_FREQ 32000000   //Only used by the delay-functions
#define onboard_led  PORTBbits.RB7
#define onboard_button  PORTBbits.RB6
#define pwm_1_dc CCPR2L
#define pwm_2_dc CCPR5L



#pragma config FOSC = HS1    //extern oszilator 4-16mhz
#pragma config PLLCFG = ON   //PLL enable, CPU=32 MHz
#pragma config PWRTEN = ON   //PowerOnTimer on
#pragma config WDTEN = OFF   //Watchdog Timer off
#pragma config XINST = OFF   //Extended Instruction Set off, no support by free compiler
#pragma config SOSCSEL = DIG //PORTC.0 and PORTC.1 digital I/O, not second oszilator

int received_chars;
char received[20];
int values_received;


void delay_one_second (void)    //make a break for one second
{
    int i;
    for(i=0; i<50; i++) {
     __delay_ms(20);   //inline delay max. 20ms, so repeat it 50 times for one second
    }
}

//needed by printf, used to configure the destination
void putch(char data) {
     while( ! TXIF)      // check buffer
     continue;      // wait till ready
     TXREG = data;      // send data
    } 

void main(void) {

    //config outputs
    LATBbits.LATB7 = 0;TRISBbits.TRISB7 = 0; //onboard_led
    LATCbits.LATC2 = 0;TRISCbits.TRISC2 = 0; //PWM 1 (ccp2)
    LATBbits.LATB5 = 0;TRISBbits.TRISB5 = 0; //PWM 2 (ccp5)
 
    //config inputs 
    LATBbits.LATB6 = 0;TRISBbits.TRISB6 = 1; //onboard_button
    
     //CONFIG ADCs
    ANCON0 = 0b00000111;     // Just AN0 AN1 AN2 are an analog input, all other pins are digital
    ANCON1 = 0b00000000;
    ADCON1 = 0b00000000;     //No special triggers, Ref is Vdd and Vss
    ADCON2 = 0b10111100;     //result is right justified, 8 Tads, Clock = 32Mhz / 4
        
    //CONFIG PWMs
    CCPTMRS = 0x00;  //All PWMs uses TMR2
    CCPR2H = 0xff; //Setup CCP2 Period register
    CCPR5H = 0xff; //Setup CCP5 Period register
    pwm_1_dc = 0; //setup duty-cycle to 0%
    pwm_2_dc = 0; //setup duty-cycle to 0%
    CCP2CON = 0b00001100; //Setup CCP2 for PWM-Mode
    CCP5CON = 0b00001100; //Setup CCP2 for PWM-Mode   
     //Timer2 for PWM
    T2CON = 0b00000110; //Config Timer2
  
    Open1USART     //Init USART 1
    (USART_TX_INT_OFF &     //Send-Interrupt off
     USART_RX_INT_ON &     //Receivce-Interrupt on
     USART_ASYNCH_MODE &
     USART_EIGHT_BIT &
     USART_CONT_RX &
     USART_BRGH_HIGH, 16);      //Baud 16 => 115.200
   
    //enable Interrupts
        RCONbits.IPEN = 1; //enable High/Low-Prio
    ei(); // master switch to enable interrupt
    INTCONbits.GIE_GIEH = 1; //disable all High-Prio-Interrupts
    INTCONbits.PEIE_GIEL = 1; //enable all Low-Prio-Interrupts
    
    
    int ad1, ad2;
    
    while(1)
    {
    ADCON0 = 0b00000001;     //Select AD-Channel 0
    ConvertADC();     //start AD-Conversion
    while(BusyADC());     //wait for conversion to finish
    ad1 = ((ADRESH)<<8)|(ADRESL);    //read in the result
    
    ADCON0 = 0b00000101;     //Select AD-Channel 1
    ConvertADC();     //start AD-Conversion
    while(BusyADC());     //wait for conversion to finish
    ad2 = ((ADRESH)<<8)|(ADRESL);    //read in the result
   
    if (ad1 < 0) ad1 = 0;
    if (ad2 < 0) ad2 = 0;
    
    printf("A;%i;%i;0\n",ad1,ad2);
   
    if (values_received == 1)
    {
       pwm_1_dc = ((received[1]-48)*100 + (received[2]-48)*10 + (received[3]-48));
       pwm_2_dc = ((received[5]-48)*100 + (received[6]-48)*10 + (received[7]-48));
        values_received = 0;
    }
    __delay_ms(15);
    }
    return;
}

void interrupt high_isr(void) //High priority interrupts
{
    if (PIR1bits.RC1IF == 1)
    {
        char temp;
        temp = getc1USART();
        if (temp == 'A') received_chars = 0;
        else received_chars++;
        if (received_chars >= 19) received_chars = 18;
        received[received_chars] = temp;
        if (received_chars == 12) values_received = 1;     
        PIR1bits.RC1IF = 0;
    }
	
}
